@@ -12,9 +12,12 @@ const Lang = imports.lang;
 const MainWindow = imports.mainWindow;
 const PasteManager = imports.pasteManager;
 const Utils = imports.utils;
+const NetworksManager = imports.networksManager;
 
 const MAX_RETRIES = 3;
 
+const IRC_SCHEMA_REGEX = /^(irc?:\/\/)([\da-z\.-]+):?(\d+)?\/(?:%23)?([\w\.\+-]+)/i;
+
 const ConnectionError = {
     CANCELLED: Tp.error_get_dbus_name(Tp.Error.CANCELLED),
     ALREADY_CONNECTED: Tp.error_get_dbus_name(Tp.Error.ALREADY_CONNECTED)
@@ -23,9 +26,11 @@ const ConnectionError = {
 const Application = new Lang.Class({
     Name: 'Application',
     Extends: Gtk.Application,
+    Signals: { 'prepare-shutdown': {} },
 
     _init: function() {
-        this.parent({ application_id: 'org.gnome.Polari' });
+        this.parent({ application_id: 'org.gnome.Polari',
+                      flags: Gio.ApplicationFlags.HANDLES_OPEN });
 
         GLib.set_application_name('Polari');
         this._window = null;
@@ -38,6 +43,7 @@ const Application = new Lang.Class({
         this._chatroomManager = ChatroomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
         this._networkMonitor = Gio.NetworkMonitor.get_default();
+        this._networksManager = NetworksManager.getDefault();
 
         this._accountsMonitor.connect('account-removed', Lang.bind(this,
             function(am, account) {
@@ -51,16 +57,9 @@ const Application = new Lang.Class({
         this.commandOutputQueue = new AppNotifications.CommandOutputQueue();
 
         let actionEntries = [
-          { name: 'room-menu',
-            activate: Lang.bind(this, this._onToggleAction),
-            state: GLib.Variant.new('b', false) },
           { name: 'show-join-dialog',
             activate: Lang.bind(this, this._onShowJoinDialog),
             accels: ['<Primary>n'] },
-          { name: 'show-message-user-dialog',
-            activate: Lang.bind(this, this._onShowMessageUserDialog),
-            create_hook: Lang.bind(this, this._accountActionsCreateHook),
-            accels: ['<Primary>m'] },
           { name: 'join-room',
             activate: Lang.bind(this, this._onJoinRoom),
             parameter_type: GLib.VariantType.new('(ssu)') },
@@ -143,7 +142,7 @@ const Application = new Lang.Class({
                 function() {
                     for (let id in this._pendingRequests)
                         this._pendingRequests[id].cancellable.cancel();
-                    this.emitJS('prepare-shutdown');
+                    this.emit('prepare-shutdown');
             }));
             this._window.show_all();
 
@@ -152,6 +151,117 @@ const Application = new Lang.Class({
         this._window.present();
     },
 
+    vfunc_open: function(files) {
+        this.activate();
+
+        let time = Utils.getTpEventTime();
+        let uris = files.map(function(f) { return f.get_uri(); });
+
+        let quark = Tp.AccountManager.get_feature_quark_core();
+        if (this._accountsMonitor.accountManager.is_prepared(quark))
+            this._openURIs(uris, time);
+        else
+            this._accountsMonitor.connect('account-manager-prepared', Lang.bind(this,
+                function(mon) {
+                    this._openURIs(uris, time);
+                }));
+    },
+
+    _openURIs: function(uris, time) {
+        let map = {};
+
+        this._accountsMonitor.dupAccounts().forEach(function(a) {
+            if (!a.enabled)
+                return;
+
+            let params = a.dup_parameters_vardict().deep_unpack();
+            map[a.get_object_path()] = {
+                server: params.server.deep_unpack(),
+                service: a.service
+            };
+        });
+
+        let joinAction = this.lookup_action('join-room');
+        uris.forEach(Lang.bind(this, function(uri) {
+            let [success, server, port, room] = this._parseURI(uri);
+            if (!success)
+                return;
+
+            // When launching Polari via a URL, there is a potential race
+            // condition between networksManager and AccountsManager here. If
+            // AccountsManager finishes first, networksManager's list of
+            // servers would be empty and hence no matches would be found.
+            let matchedId = this._networksManager.findByServer(server);
+            let matches = Object.keys(map).filter(function(a) {
+                return GLib.ascii_strcasecmp(map[a].server, server) == 0 ||
+                       map[a].service == matchedId;
+            });
+
+            if (matches.length)
+                joinAction.activate(new GLib.Variant('(ssu)',
+                                [matches[0], '#' + room, time]));
+            else
+                this._createAccount(matchedId, server, port,
+                    function(a) {
+                        if (a)
+                            joinAction.activate(new GLib.Variant('(ssu)',
+                                            [a.get_object_path(),
+                                             '#' + room, time]));
+                    });
+        }));
+    },
+
+    _parseURI: function(uri) {
+        let server, port, room;
+        let success = false;
+        try {
+            [,, server, port, room] = uri.match(IRC_SCHEMA_REGEX);
+            success = true;
+        } catch(e) {
+            let label = _("Failed to open link");
+            let n = new AppNotifications.MessageNotification(label,
+                                                             'dialog-error-symbolic');
+            this.notificationQueue.addNotification(n);
+        }
+
+        return [success, server, port, room];
+    },
+
+    _createAccount: function(id, server, port, callback) {
+        let params, name;
+
+        if (id) {
+            params = this._networksManager.getNetworkDetails(id);
+            name = this._networksManager.getNetworkName(id);
+        } else {
+            params = {
+                'account': new GLib.Variant('s', GLib.get_user_name()),
+                'server': new GLib.Variant('s', server),
+                'port': new GLib.Variant('u', port ? port : 6667),
+                'use-ssl': new GLib.Variant('b', (port == 6697)),
+            };
+            name = server;
+        }
+
+        let req = new Tp.AccountRequest({ account_manager: Tp.AccountManager.dup(),
+                                          connection_manager: 'idle',
+                                          protocol: 'irc',
+                                          display_name: name });
+        req.set_enabled(true);
+
+        if (id)
+            req.set_service(id);
+
+        for (let prop in params)
+            req.set_parameter(prop, params[prop]);
+
+        req.create_account_async(Lang.bind(this,
+            function(r, res) {
+                let account = req.create_account_finish(res);
+                callback(account);
+            }));
+    },
+
     _updateAccountAction: function(action) {
         action.enabled = this._accountsMonitor.dupAccounts().filter(
             function(a) {
@@ -159,14 +269,6 @@ const Application = new Lang.Class({
             }).length > 0;
     },
 
-    _accountActionsCreateHook: function(action) {
-        this._accountsMonitor.connect('accounts-changed', Lang.bind(this,
-            function() {
-                this._updateAccountAction(action);
-            }));
-        this._updateAccountAction(action);
-    },
-
     _leaveRoomCreateHook: function(action) {
         this._chatroomManager.connect('active-changed', Lang.bind(this,
             function() {
@@ -196,10 +298,6 @@ const Application = new Lang.Class({
         this._window.showJoinRoomDialog();
     },
 
-    _onShowMessageUserDialog: function() {
-        this._window.showMessageUserDialog();
-    },
-
     _savedChannelIndex: function(savedChannels, account, channel) {
         let accountPath = account.get_object_path();
         let matchChannel = channel.toLowerCase();
@@ -482,7 +580,7 @@ const Application = new Lang.Class({
             comments: _("An Internet Relay Chat Client for GNOME"),
             copyright: 'Copyright © 2013-2015 The Polari authors',
             license_type: Gtk.License.GPL_2_0,
-            logo_icon_name: 'polari',
+            logo_icon_name: 'org.gnome.Polari',
             version: pkg.version,
             website_label: _("Learn more about Polari"),
             website: 'https://wiki.gnome.org/Apps/Polari',
@@ -503,4 +601,3 @@ const Application = new Lang.Class({
         this._window.destroy();
     }
 });
-Utils.addJSSignalMethods(Application.prototype);
