@@ -11,7 +11,6 @@ const ChatroomManager = imports.chatroomManager;
 const JoinDialog = imports.joinDialog;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
-const MessageDialog = imports.messageDialog;
 const RoomList = imports.roomList;
 const RoomStack = imports.roomStack;
 const UserList = imports.userList;
@@ -59,6 +58,7 @@ const FixedSizeFrame = new Lang.Class({
             return;
         this._height = height;
         this.notify('height');
+        this.set_size_request(this._width, this._height);
         this._queueRedraw();
     },
 
@@ -72,21 +72,18 @@ const FixedSizeFrame = new Lang.Class({
 
         this._width = width;
         this.notify('width');
+        this.set_size_request(this._width, this._height);
         this._queueRedraw();
     },
 
     vfunc_get_preferred_width_for_height: function(forHeight) {
-        if (this._width < 0)
-            return this.parent(forHeight);
-        else
-            return [this._width, this._width];
+        let [min, nat] = this.parent(forHeight);
+        return [min, this._width < 0 ? nat : this._width];
     },
 
     vfunc_get_preferred_height_for_width: function(forWidth) {
-        if (this._height < 0)
-            return this.parent(forWidth);
-        else
-            return [this._height, this._height];
+        let [min, nat] = this.parent(forWidth);
+        return [min, this._height < 0 ? nat : this._height];
     }
 });
 
@@ -96,7 +93,7 @@ const MainWindow = new Lang.Class({
     Template: 'resource:///org/gnome/Polari/ui/main-window.ui',
     InternalChildren: ['titlebarRight',
                        'titlebarLeft',
-                       'joinMenuButton',
+                       'joinButton',
                        'showUserListButton',
                        'userListPopover',
                        'roomListRevealer',
@@ -149,7 +146,7 @@ const MainWindow = new Lang.Class({
                                       GObject.BindingFlags.SYNC_CREATE);
 
         // Make sure user-list button is at least as wide as icon buttons
-        this._joinMenuButton.connect('size-allocate', Lang.bind(this,
+        this._joinButton.connect('size-allocate', Lang.bind(this,
             function(w, rect) {
                 let width = rect.width;
                 Mainloop.idle_add(Lang.bind(this, function() {
@@ -299,11 +296,6 @@ const MainWindow = new Lang.Class({
         dialog.show();
     },
 
-    showMessageUserDialog: function() {
-        let dialog = new MessageDialog.MessageDialog({ transient_for: this });
-        dialog.show();
-    },
-
     _updateUserListLabel: function() {
         let numMembers = 0;
 
